/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.glassfish.openesb.pojose.api;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import org.glassfish.openesb.pojose.api.annotation.ConsumerEndpoint;
import org.glassfish.openesb.pojose.api.res.Context;

/**
 * Instance of Consumer used to consume Service. 
 * Instance of this class will be injected into JBI Provider field when annotaed 
 * with ConsumerEndpoint. Instance can also be obtained from the Context object.
 * Note this class instance is not thread safe and should not be used concurrently.
 * 
 * @author gmpatil
 * @see Context
 * @see ConsumerEndpoint
 */
public interface Consumer {
    //public enum MessageObjectType {String, Document, Element, TextNode, NormalizedMessage};    
    public enum MessageObjectType {String, Node, Source, NormalizedMessage};

    /**
     * Sets ServiceEndpoint.
     *
     * @param se ServiceEndpoint
     */
    public void setServiceEndpoint(ServiceEndpoint se);

    /**
     * Returns ServiceEndpoint instance associated with this Consumer.
     *
     * @return ServiceEndpoint
     */
    public ServiceEndpoint getServiceEndpoint();    

    /**
     * Sets the service QName instance. ServiceEndpoint instance if set will take
     * precedence over service QName.
     * 
     * @param svc Service QName.
     */    
    public void setService(QName svc);

    /**
     * Return QName of the Service associated with this consumer.
     *
     * @return QName of the service
     */
    public QName getService();

    /**
     * Sets the default interface QName.
     * 
     * @param intf Interface QName
     */    
    public void setInterface(QName intf);

    /**
     * Get the default interface QName associated with this consumer.
     * 
     * @return QName of the default interface.
     */
    public QName getInterface();
    
    /**
     * Input message type to be used inside WSDL 1.1 wrapper JBI message.
     * This is used only during implicit WSDL 1.1 message wrapping.
     * 
     * @param inpt QName of input message type.
     */
    public void setDefaultInputMessageType(QName inpt);

    /**
     * Operation name to be set on MessageExchange created implicitly by Consumer.
     * 
     * @param  opr QName of operation.
     */    
    public void setDefaultOperationName(QName opr);    

    /**
     * Sends the MessageExchange in synch mode. User should take care to use 
     * appropriate MessageExchange, i.e either InOnly or InOut.
     * 
     * Any MessagingException from delivery channel is wrapped in POJOError.
     * 
     * Uses the available default ServiceEndpoint, Service or Interface in order 
     * if all of the above is null in the passed MessageExchange.
     * 
     * Default Operation name is set if it is null on the passed MessageExchange.
     * 
     * @param me MessageExchange 
     * @return true if the exchange has been processed and returned by the servicing component, false otherwise.
     * @throws org.glassfish.openesb.pojose.api.POJOError any MessagingException is also wrapped in this object.
     */
    public boolean sendSynch(MessageExchange me) throws MessageException;

    /**
     * Sends the ME is asynch mode.
     * Currently asynch consumption is not supported yet by POJO SE.
     * So this method should be only used to send the status in asynch mode
     * as needed by JBI runtime/NMR.<br>
     * Sending done, error status in <b>synch</b> mode will result in exception from JBI runtime.
     * <br>
     * JBIMR0024: SendSynch operation not legal in current state.
     * <br>
     * @param me MessageExchange
     * @throws org.glassfish.openesb.pojose.api.ErrorMessage any MessagingException is also wrapped in this object.
     */
    public void send(MessageExchange me) throws ErrorMessage;
    /**
     * Sends the message to given ServiceEndpoint.
     * Any MessagingException will be wrapped in Error to the caller, 
     * including when Delivery channel fails to process the message by returning 
     * false.
     * 
     * Implicit object conversion is done as below.
     * Instance of NormalizedMessage is sent as is. 
     * Instance of Source, Node, String is wrapped with WSDL 1.1 JBI wrapper 
     * message elements.
     * @param msg instance of type String, Node, Source or NormalizedMessage.
     * @throws org.glassfish.openesb.pojose.api.Error
     */
    public void sendSynchInOnly(Object msg) throws ErrorMessage;
    
    /**
     * Sends the message to given ServiceEndpoint.
     * Any MessagingException will be wrapped in POJOError to the caller 
     * including when Delivery channel fails to process the message by returning 
     * false.
     * 
     * Implicit input object conversion is done as below.
     * Instance of NormalizedMessage is sent as is. 
     * Instance of Source, Node and String is wrapped with WSDL 1.1 JBI wrapper 
     * message elements before sending it as source in a NormalizedMessage.
     * 
     * Output or returned object conversion is done based on the given 
     * MessageObjectType. Implicit WSDL 1.1 JBI message unwrapping is done when 
     * returned message is of type String, Node and Source.
     * 
     * @param inMsg instance of type String, Node, Source or NormalizedMessage.
     * @throws org.glassfish.openesb.pojose.api.Error
     */    
    public Object sendSynchInOut(Object inMsg, MessageObjectType outputType) throws MessageException;
}
